@@ -4,11 +4,11 @@ from .core import istask
 from .context import set_options
 from .async import get_sync as get
 try:
-    from .delayed import do, delayed, value
+    from .delayed import do, delayed
 except ImportError:
     pass
 try:
-    from .base import visualize, compute
+    from .base import visualize, compute, persist
 except ImportError:
     pass
 
