@@ -13,10 +13,12 @@ from itertools import product, chain, count
 from operator import getitem, add, mul, itemgetter
 
 import numpy as np
-from toolz import merge, accumulate, reduce
+import toolz
+from toolz import accumulate, reduce
 
 from ..base import tokenize
 from .core import concatenate3, Array, normalize_chunks
+from .. import sharedict
 
 
 def cumdims_label(chunks, const):
@@ -342,10 +344,14 @@ def find_merge_rechunk(old_chunks, new_chunks, block_size_limit):
     # block_size_limit.  This is in effect a knapsack problem, except with
     # multiplicative values and weights.  Just use a greedy algorithm
     # by trying dimensions in decreasing value / weight order.
-    sorted_candidates = sorted(
-        merge_candidates,
-        key=lambda k: np.log(graph_size_effect[k]) / np.log(block_size_effect[k]),
-    )
+    def key(k):
+        gse = graph_size_effect[k]
+        bse = block_size_effect[k]
+        if bse == 1:
+            bse = 1 + 1e-9
+        return np.log(gse) / np.log(bse)
+
+    sorted_candidates = sorted(merge_candidates, key=key)
 
     largest_block_size = reduce(mul, old_largest_width)
 
@@ -519,7 +525,7 @@ def _compute_rechunk(x, chunks):
     assert new_idx == tuple(len(c) - 1 for c in chunks)
     del old_blocks, new_index
 
-    x2 = merge(x.dask, x2, intermediates)
+    x2 = sharedict.merge(x.dask, (merge_temp_name, toolz.merge(x2, intermediates)))
     return Array(x2, merge_temp_name, chunks, dtype=x.dtype)
 
 
