@@ -1,9 +1,9 @@
 from __future__ import absolute_import, division, print_function
 
-from collections import OrderedDict
+from collections import OrderedDict, Iterator
+import copy
 from functools import partial
 from hashlib import md5
-from operator import attrgetter
 import pickle
 import os
 import uuid
@@ -11,9 +11,11 @@ import uuid
 from toolz import merge, groupby, curry, identity
 from toolz.functoolz import Compose
 
-from .compatibility import bind_method, unicode
+from .compatibility import bind_method, unicode, PY3
 from .context import _globals
+from .core import flatten
 from .utils import Dispatch
+from .sharedict import ShareDict
 
 __all__ = ("Base", "compute", "normalize_token", "tokenize", "visualize")
 
@@ -61,8 +63,21 @@ class Base(object):
         return visualize(self, filename=filename, format=format,
                          optimize_graph=optimize_graph, **kwargs)
 
+    def persist(self, **kwargs):
+        """ Persist this dask collection into memory
+
+        See ``dask.base.persist`` for full docstring
+        """
+        (result,) = persist(self, **kwargs)
+        return result
+
     def compute(self, **kwargs):
-        """Compute several dask collections at once.
+        """ Compute this dask collection
+
+        This turns a lazy Dask collection into its in-memory equivalent.
+        For example a Dask.array turns into a NumPy array and a Dask.dataframe
+        turns into a Pandas dataframe.  The entire dataset must fit into memory
+        before calling this operation.
 
         Parameters
         ----------
@@ -76,12 +91,13 @@ class Base(object):
         kwargs
             Extra keywords to forward to the scheduler ``get`` function.
         """
-        return compute(self, **kwargs)[0]
+        (result,) = compute(self, traverse=False, **kwargs)
+        return result
 
     @classmethod
     def _get(cls, dsk, keys, get=None, **kwargs):
         get = get or _globals['get'] or cls._default_get
-        dsk2 = cls._optimize(dsk, keys, **kwargs)
+        dsk2 = cls._optimize(dict(dsk), keys, **kwargs)
         return get(dsk2, keys, **kwargs)
 
     @classmethod
@@ -125,9 +141,15 @@ def compute(*args, **kwargs):
     Parameters
     ----------
     args : object
-        Any number of objects. If the object is a dask collection, it's
-        computed and the result is returned. Otherwise it's passed through
-        unchanged.
+        Any number of objects. If it is a dask object, it's computed and the
+        result is returned. By default, python builtin collections are also
+        traversed to look for dask objects (for more information see the
+        ``traverse`` keyword). Non-dask arguments are passed through unchanged.
+    traverse : bool, optional
+        By default dask traverses builtin python collections looking for dask
+        objects passed to ``compute``. For large collections this can be
+        expensive. If none of the arguments contain any dask objects, set
+        ``traverse=False`` to avoid doing this traversal.
     get : callable, optional
         A scheduler ``get`` function to use. If not provided, the default is
         to check the global settings first, and then fall back to defaults for
@@ -146,14 +168,25 @@ def compute(*args, **kwargs):
     >>> b = da.arange(10, chunks=2).mean()
     >>> compute(a, b)
     (45, 4.5)
+
+    By default, dask objects inside python collections will also be computed:
+
+    >>> compute({'a': a, 'b': b, 'c': 1})  # doctest: +SKIP
+    ({'a': 45, 'b': 4.5, 'c': 1},)
     """
+    from dask.delayed import delayed
+    traverse = kwargs.pop('traverse', True)
+    if traverse:
+        args = tuple(delayed(a)
+                     if isinstance(a, (list, set, tuple, dict, Iterator))
+                     else a for a in args)
+
+    optimize_graph = kwargs.pop('optimize_graph', True)
     variables = [a for a in args if isinstance(a, Base)]
     if not variables:
         return args
 
     get = kwargs.pop('get', None) or _globals['get']
-    optimizations = (kwargs.pop('optimizations', None) or
-                     _globals.get('optimizations', []))
 
     if not get:
         get = variables[0]._default_get
@@ -163,18 +196,7 @@ def compute(*args, **kwargs):
                              "scheduler `get` function using either "
                              "the `get` kwarg or globally with `set_options`.")
 
-    if kwargs.get('optimize_graph', True):
-        groups = groupby(attrgetter('_optimize'), variables)
-        groups = {opt: [merge([v.dask for v in val]),
-                        [v._keys() for v in val]]
-                  for opt, val in groups.items()}
-        for opt in optimizations:
-            groups = {k: [opt(dsk, keys), keys]
-                      for k, (dsk, keys) in groups.items()}
-        dsk = merge([opt(dsk, keys, **kwargs)
-                    for opt, (dsk, keys) in groups.items()])
-    else:
-        dsk = merge(var.dask for var in variables)
+    dsk = collections_to_dsk(variables, optimize_graph, **kwargs)
     keys = [var._keys() for var in variables]
     results = get(dsk, keys, **kwargs)
 
@@ -229,7 +251,8 @@ def visualize(*args, **kwargs):
     optimize_graph = kwargs.pop('optimize_graph', False)
     from dask.dot import dot_graph
     if optimize_graph:
-        dsks.extend([arg._optimize(arg.dask, arg._keys()) for arg in args])
+        dsks.extend([arg._optimize(dict(arg.dask), arg._keys())
+                     for arg in args])
     else:
         dsks.extend([arg.dask for arg in args])
     dsk = merge(dsks)
@@ -277,7 +300,12 @@ def normalize_ordered_dict(d):
     return type(d).__name__, normalize_token(list(d.items()))
 
 
-@normalize_token.register((tuple, list, set))
+@normalize_token.register(set)
+def normalize_set(s):
+    return normalize_token(sorted(s, key=str))
+
+
+@normalize_token.register((tuple, list))
 def normalize_seq(seq):
     return type(seq).__name__, list(map(normalize_token, seq))
 
@@ -342,7 +370,8 @@ def register_numpy():
             try:
                 data = md5('-'.join(x.flat).encode('utf-8')).hexdigest()
             except TypeError:
-                data = md5(b'-'.join([str(item).encode() for item in x.flat])).hexdigest()
+                data = md5(b'-'.join([unicode(item).encode('utf-8') for item in
+                                      x.flat])).hexdigest()
         else:
             try:
                 data = md5(x.ravel().view('i1').data).hexdigest()
@@ -366,3 +395,157 @@ def tokenize(*args, **kwargs):
     if kwargs:
         args = args + (kwargs,)
     return md5(str(tuple(map(normalize_token, args))).encode()).hexdigest()
+
+
+def collections_to_dsk(collections, optimize_graph=True, **kwargs):
+    """
+    Convert many collections into a single dask graph, after optimization
+    """
+    optimizations = (kwargs.pop('optimizations', None) or
+                     _globals.get('optimizations', []))
+    if optimize_graph:
+        groups = groupby(lambda x: x._optimize, collections)
+        groups = {opt: _extract_graph_and_keys(val)
+                  for opt, val in groups.items()}
+        for opt in optimizations:
+            groups = {k: [opt(dict(dsk), keys), keys]
+                      for k, (dsk, keys) in groups.items()}
+        dsk = merge([opt(dsk, keys, **kwargs)
+                     for opt, (dsk, keys) in groups.items()])
+    else:
+        dsk = merge(dict(c.dask) for c in collections)
+
+    return dsk
+
+
+def _extract_graph_and_keys(vals):
+    """Given a list of dask vals, return a single graph and a list of keys such
+    that ``get(dsk, keys)`` is equivalent to ``[v.compute() v in vals]``."""
+    dsk = {}
+    keys = []
+    for v in vals:
+        d = v.dask
+        if type(d) is ShareDict:
+            for dd in d.dicts.values():
+                dsk.update(dd)
+        else:
+            dsk.update(v.dask)
+        keys.append(v._keys())
+
+    return dsk, keys
+
+
+def redict_collection(c, dsk):
+    cc = copy.copy(c)
+    cc.dask = dsk
+    return cc
+
+
+def persist(*args, **kwargs):
+    """ Persist multiple Dask collections into memory
+
+    This turns lazy Dask collections into Dask collections with the same
+    metadata, but now with their results fully computed or actively computing
+    in the background.
+
+    For example a lazy dask.array built up from many lazy calls will now be a
+    dask.array of the same shape, dtype, chunks, etc., but now with all of
+    those previously lazy tasks either computed in memory as many small NumPy
+    arrays (in the single-machine case) or asynchronously running in the
+    background on a cluster (in the distributed case).
+
+    This function operates differently if a ``dask.distributed.Client`` exists
+    and is connected to a distributed scheduler.  In this case this function
+    will return as soon as the task graph has been submitted to the cluster,
+    but before the computations have completed.  Computations will continue
+    asynchronously in the background.  When using this function with the single
+    machine scheduler it blocks until the computations have finished.
+
+    When using Dask on a single machine you should ensure that the dataset fits
+    entirely within memory.
+
+    Examples
+    --------
+    >>> df = dd.read_csv('/path/to/*.csv')  # doctest: +SKIP
+    >>> df = df[df.name == 'Alice']  # doctest: +SKIP
+    >>> df['in-debt'] = df.balance < 0  # doctest: +SKIP
+    >>> df = df.persist()  # triggers computation  # doctest: +SKIP
+
+    >>> df.value().min()  # future computations are now fast  # doctest: +SKIP
+    -10
+    >>> df.value().max()  # doctest: +SKIP
+    100
+
+    >>> from dask import persist  # use persist function on multiple collections
+    >>> a, b = persist(a, b)  # doctest: +SKIP
+
+    Parameters
+    ----------
+    *args: Dask collections
+    get : callable, optional
+        A scheduler ``get`` function to use. If not provided, the default
+        is to check the global settings first, and then fall back to
+        the collection defaults.
+    optimize_graph : bool, optional
+        If True [default], the graph is optimized before computation.
+        Otherwise the graph is run as is. This can be useful for debugging.
+    **kwargs
+        Extra keywords to forward to the scheduler ``get`` function.
+
+    Returns
+    -------
+    New dask collections backed by in-memory data
+    """
+    collections = [a for a in args if isinstance(a, Base)]
+    if not collections:
+        return args
+
+    try:
+        from distributed.client import default_client
+    except ImportError:
+        pass
+    else:
+        try:
+            client = default_client()
+        except ValueError:
+            pass
+        else:
+            collections = client.persist(collections, **kwargs)
+            if isinstance(collections, list):  # distributed is inconsistent here
+                collections = tuple(collections)
+            else:
+                collections = (collections,)
+            results_iter = iter(collections)
+            return tuple(a if not isinstance(a, Base)
+                         else next(results_iter)
+                         for a in args)
+
+    optimize_graph = kwargs.pop('optimize_graph', True)
+
+    get = kwargs.pop('get', None) or _globals['get']
+
+    if not get:
+        get = collections[0]._default_get
+        if not all(a._default_get == get for a in collections):
+            raise ValueError("Compute called on multiple collections with "
+                             "differing default schedulers. Please specify a "
+                             "scheduler `get` function using either "
+                             "the `get` kwarg or globally with `set_options`.")
+
+    dsk = collections_to_dsk(collections, optimize_graph, **kwargs)
+    keys = list(flatten([var._keys() for var in collections]))
+    results = get(dsk, keys, **kwargs)
+
+    d = dict(zip(keys, results))
+
+    result = [redict_collection(c, {k: d[k]
+                                    for k in flatten(c._keys())})
+              for c in collections]
+    results_iter = iter(result)
+    return tuple(a if not isinstance(a, Base)
+                 else next(results_iter)
+                 for a in args)
+
+
+if PY3:
+    Base.persist.__doc__ = persist.__doc__
