@@ -318,7 +318,7 @@ def test_groupby_index_array():
 def test_groupby_set_index():
     df = tm.makeTimeDataFrame()
     ddf = dd.from_pandas(df, npartitions=2)
-    pytest.raises(NotImplementedError,
+    pytest.raises(TypeError,
                   lambda: ddf.groupby(df.index.month, as_index=False))
 
 
@@ -965,9 +965,103 @@ def test_split_out_multi_column_groupby():
     assert_eq(result, expected, check_dtype=False)
 
 
+def test_groupby_split_out_num():
+    # GH 1841
+    ddf = dd.from_pandas(pd.DataFrame({'A': [1, 1, 2, 2],
+                                       'B': [1, 2, 3, 4]}),
+                         npartitions=2)
+    assert ddf.groupby('A').sum().npartitions == 1
+    assert ddf.groupby('A').sum(split_out=2).npartitions == 2
+    assert ddf.groupby('A').sum(split_out=3).npartitions == 3
+
+    with pytest.raises(TypeError):
+        # groupby doesn't adcept split_out
+        ddf.groupby('A', split_out=2)
+
+
+def test_groupby_not_supported():
+    ddf = dd.from_pandas(pd.DataFrame({'A': [1, 1, 2, 2],
+                                       'B': [1, 2, 3, 4]}),
+                         npartitions=2)
+    with pytest.raises(TypeError):
+        ddf.groupby('A', axis=1)
+    with pytest.raises(TypeError):
+        ddf.groupby('A', level=1)
+    with pytest.raises(TypeError):
+        ddf.groupby('A', as_index=False)
+    with pytest.raises(TypeError):
+        ddf.groupby('A', sort=False)
+    with pytest.raises(TypeError):
+        ddf.groupby('A', group_keys=False)
+    with pytest.raises(TypeError):
+        ddf.groupby('A', squeeze=True)
+
+
 def test_groupby_numeric_column():
     df = pd.DataFrame({'A' : ['foo', 'foo', 'bar'], 0: [1,2,3]})
     ddf = dd.from_pandas(df, npartitions=3)
 
     assert_eq(ddf.groupby(ddf.A)[0].sum(),
               df.groupby(df.A)[0].sum())
+
+
+@pytest.mark.parametrize('sel', ['c', 'd', ['c', 'd']])
+@pytest.mark.parametrize('key', ['a', ['a', 'b']])
+@pytest.mark.parametrize('func', ['cumsum', 'cumprod', 'cumcount'])
+def test_cumulative(func, key, sel):
+    df = pd.DataFrame({'a': [1, 2, 6, 4, 4, 6, 4, 3, 7] * 6,
+                       'b': [4, 2, 7, 3, 3, 1, 1, 1, 2] * 6,
+                       'c': np.random.randn(54),
+                       'd': np.random.randn(54)},
+                      columns=['a', 'b', 'c', 'd'])
+    df.iloc[[-18, -12, -6], -1] = np.nan
+    ddf = dd.from_pandas(df, npartitions=10)
+
+    g, dg = [d.groupby(key)[sel] for d in (df, ddf)]
+    assert_eq(getattr(g, func)(), getattr(dg, func)())
+
+
+@pytest.mark.parametrize('func', ['cumsum', 'cumprod'])
+def test_cumulative_axis1(func):
+    df = pd.DataFrame({'a': [1, 2, 6, 4, 4, 6, 4, 3, 7] * 2,
+                       'b': np.random.randn(18),
+                       'c': np.random.randn(18)})
+    df.iloc[-6, -1] = np.nan
+    ddf = dd.from_pandas(df, npartitions=4)
+    assert_eq(getattr(df.groupby('a'), func)(axis=1),
+              getattr(ddf.groupby('a'), func)(axis=1))
+
+
+def test_groupby_unaligned_index():
+    df = pd.DataFrame({'a': np.random.randint(0, 10, 50),
+                       'b': np.random.randn(50),
+                       'c': np.random.randn(50)})
+    ddf = dd.from_pandas(df, npartitions=5)
+    filtered = df[df.b < 0.5]
+    dfiltered = ddf[ddf.b < 0.5]
+
+    ddf_group = dfiltered.groupby(ddf.a)
+    ds_group = dfiltered.b.groupby(ddf.a)
+
+    bad = [ddf_group.mean(),
+           ddf_group.var(),
+           ddf_group.b.nunique(),
+           ddf_group.get_group(0),
+           ds_group.mean(),
+           ds_group.var(),
+           ds_group.nunique(),
+           ds_group.get_group(0)]
+
+    for obj in bad:
+        with pytest.raises(ValueError):
+            obj.compute()
+
+    def add1(x):
+        return x + 1
+
+    df_group = filtered.groupby(df.a)
+    good = [(ddf_group.apply(add1, meta=ddf), df_group.apply(add1)),
+            (ddf_group.b.apply(add1, meta=ddf.b), df_group.b.apply(add1))]
+
+    for (res, sol) in good:
+        assert_eq(res, sol)
