@@ -1,4 +1,3 @@
-
 import numpy as np
 import pandas as pd
 import pandas.util.testing as tm
@@ -6,7 +5,7 @@ import pytest
 
 import dask.dataframe as dd
 
-from dask.dataframe.utils import assert_eq, PANDAS_ge_0190
+from dask.dataframe.utils import assert_eq, make_meta
 
 
 @pytest.mark.parametrize('data', [
@@ -91,6 +90,20 @@ def test_get_dummies_errors():
         ds = dd.from_pandas(s, 2)
         dd.get_dummies(ds)
 
+    # unknown categories
+    df = pd.DataFrame({'x': list('abcbc'), 'y': list('bcbcb')})
+    ddf = dd.from_pandas(df, npartitions=2)
+    ddf._meta = make_meta({'x': 'category', 'y': 'category'})
+
+    with pytest.raises(NotImplementedError):
+        dd.get_dummies(ddf)
+
+    with pytest.raises(NotImplementedError):
+        dd.get_dummies(ddf, columns=['x', 'y'])
+
+    with pytest.raises(NotImplementedError):
+        dd.get_dummies(ddf.x)
+
 
 @pytest.mark.parametrize('aggfunc', ['mean', 'sum', 'count'])
 def test_pivot_table(aggfunc):
@@ -107,11 +120,7 @@ def test_pivot_table(aggfunc):
         # dask result cannot be int64 dtype depending on divisions because of NaN
         exp = exp.astype(np.float64)
 
-    if PANDAS_ge_0190:
-        assert_eq(res, exp)
-    else:
-        # because of a pandas 0.18.x bug, categorical dtype is not preserved
-        assert_eq(res, exp, check_names=False, check_column_type=False)
+    assert_eq(res, exp)
 
     # method
     res = ddf.pivot_table(index='A', columns='C', values='B',
@@ -121,11 +130,7 @@ def test_pivot_table(aggfunc):
     if aggfunc == 'count':
         # dask result cannot be int64 dtype depending on divisions because of NaN
         exp = exp.astype(np.float64)
-    if PANDAS_ge_0190:
-        assert_eq(res, exp)
-    else:
-        # because of a pandas 0.18.x bug, categorical dtype is not preserved
-        assert_eq(res, exp, check_names=False, check_column_type=False)
+    assert_eq(res, exp)
 
 
 def test_pivot_table_dtype():
@@ -144,11 +149,7 @@ def test_pivot_table_dtype():
     exp = pd.pivot_table(df, index='A', columns='B',
                          values='C', aggfunc='count').astype(np.float64)
 
-    if PANDAS_ge_0190:
-        assert_eq(res, exp)
-    else:
-        # because of a pandas 0.18.x bug, categorical dtype is not preserved
-        assert_eq(res, exp, check_names=False, check_column_type=False)
+    assert_eq(res, exp)
 
 
 def test_pivot_table_errors():
@@ -174,6 +175,12 @@ def test_pivot_table_errors():
     with tm.assertRaisesRegexp(ValueError, msg):
         dd.pivot_table(ddf, index='A', columns='C', values='B', aggfunc='xx')
 
+    # unknown categories
+    ddf._meta = make_meta({'A': object, 'B': float, 'C': 'category'})
+    msg = "'columns' must have known categories"
+    with tm.assertRaisesRegexp(ValueError, msg):
+        dd.pivot_table(ddf, index='A', columns='C', values=['B'])
+
     df = pd.DataFrame({'A': np.random.choice(list('abc'), size=10),
                        'B': np.random.randn(10),
                        'C': np.random.choice(list('abc'), size=10)})
