@@ -4,15 +4,17 @@ from collections import Iterator
 from itertools import chain
 import operator
 import uuid
+import warnings
 
-from toolz import merge, unique, curry, first
+from toolz import unique, curry, first
 
-from .utils import concrete, funcname, methodcaller
-from . import base
+from . import base, threaded
 from .compatibility import apply
-from . import threaded
+from .core import quote
+from .utils import concrete, funcname, methodcaller
+from . import sharedict
 
-__all__ = ['compute', 'do', 'Delayed', 'delayed']
+__all__ = ['Delayed', 'delayed']
 
 
 def flat_unique(ls):
@@ -28,12 +30,12 @@ def unzip(ls, nout):
     return out
 
 
-def to_task_dasks(expr):
-    """Normalize a python object and extract all sub-dasks.
+def to_task_dask(expr):
+    """Normalize a python object and merge all sub-graphs.
 
     - Replace ``Delayed`` with their keys
     - Convert literals to things the schedulers can handle
-    - Extract dasks from all enclosed values
+    - Extract dask graphs from all enclosed values
 
     Parameters
     ----------
@@ -44,48 +46,47 @@ def to_task_dasks(expr):
     Returns
     -------
     task : normalized task to be run
-    dasks : list of dasks that form the dag for this task
+    dask : a merged dask graph that forms the dag for this task
 
     Examples
     --------
-
     >>> a = delayed(1, 'a')
     >>> b = delayed(2, 'b')
-    >>> task, dasks = to_task_dasks([a, b, 3])
-    >>> task # doctest: +SKIP
+    >>> task, dask = to_task_dask([a, b, 3])
+    >>> task  # doctest: +SKIP
     ['a', 'b', 3]
-    >>> dasks # doctest: +SKIP
-    [{'a': 1}, {'b': 2}]
+    >>> dict(dask)  # doctest: +SKIP
+    {'a': 1, 'b': 2}
 
-    >>> task, dasks = to_task_dasks({a: 1, b: 2})
-    >>> task # doctest: +SKIP
+    >>> task, dasks = to_task_dask({a: 1, b: 2})
+    >>> task  # doctest: +SKIP
     (dict, [['a', 1], ['b', 2]])
-    >>> dasks # doctest: +SKIP
-    [{'a': 1}, {'b': 2}]
+    >>> dict(dask)  # doctest: +SKIP
+    {'a': 1, 'b': 2}
     """
     if isinstance(expr, Delayed):
-        return expr.key, expr._dasks
+        return expr.key, expr.dask
     if isinstance(expr, base.Base):
         name = 'finalize-' + tokenize(expr, pure=True)
         keys = expr._keys()
-        dsk = expr._optimize(expr.dask, keys)
+        dsk = expr._optimize(dict(expr.dask), keys)
         dsk[name] = (expr._finalize, (concrete, keys))
-        return name, [dsk]
+        return name, dsk
     if isinstance(expr, tuple) and type(expr) != tuple:
-        return expr, []
+        return expr, {}
     if isinstance(expr, (Iterator, list, tuple, set)):
-        args, dasks = unzip((to_task_dasks(e) for e in expr), 2)
+        args, dasks = unzip((to_task_dask(e) for e in expr), 2)
         args = list(args)
-        dasks = flat_unique(dasks)
+        dsk = sharedict.merge(*dasks)
         # Ensure output type matches input type
         if isinstance(expr, (tuple, set)):
-            return (type(expr), args), dasks
+            return (type(expr), args), dsk
         else:
-            return args, dasks
+            return args, dsk
     if isinstance(expr, dict):
-        args, dasks = to_task_dasks([[k, v] for k, v in expr.items()])
-        return (dict, args), dasks
-    return expr, []
+        args, dsk = to_task_dask([[k, v] for k, v in expr.items()])
+        return (dict, args), dsk
+    return expr, {}
 
 
 def tokenize(*args, **kwargs):
@@ -107,7 +108,7 @@ def tokenize(*args, **kwargs):
 
 
 @curry
-def delayed(obj, name=None, pure=False, nout=None):
+def delayed(obj, name=None, pure=False, nout=None, traverse=True):
     """Wraps a function or object to produce a ``Delayed``.
 
     ``Delayed`` objects act as proxies for the object they wrap, but all
@@ -128,7 +129,14 @@ def delayed(obj, name=None, pure=False, nout=None):
         The number of outputs returned from calling the resulting ``Delayed``
         object. If provided, the ``Delayed`` output of the call can be iterated
         into ``nout`` objects, allowing for unpacking of results. By default
-        iteration over ``Delayed`` objects will error.
+        iteration over ``Delayed`` objects will error. Note, that ``nout=1``
+        expects ``obj``, to return a tuple of length 1, and consequently for
+        `nout=0``, ``obj`` should return an empty tuple.
+    traverse : bool, optional
+        By default dask traverses builtin python collections looking for dask
+        objects passed to ``delayed``. For large collections this can be
+        expensive. If ``obj`` doesn't contain any dask objects, set
+        ``traverse=False`` to avoid doing this traversal.
 
     Examples
     --------
@@ -246,9 +254,13 @@ def delayed(obj, name=None, pure=False, nout=None):
     if isinstance(obj, Delayed):
         return obj
 
-    task, dasks = to_task_dasks(obj)
+    if isinstance(obj, base.Base) or traverse:
+        task, dsk = to_task_dask(obj)
+    else:
+        task = quote(obj)
+        dsk = {}
 
-    if not dasks:
+    if task is obj:
         if not (nout is None or (type(nout) is int and nout >= 0)):
             raise ValueError("nout must be None or a positive integer,"
                              " got %s" % nout)
@@ -263,31 +275,21 @@ def delayed(obj, name=None, pure=False, nout=None):
     else:
         if not name:
             name = '%s-%s' % (type(obj).__name__, tokenize(task, pure=pure))
-        dasks.append({name: task})
-        return Delayed(name, dasks)
+        dsk = sharedict.merge(dsk, (name, {name: task}))
+        return Delayed(name, dsk)
 
 
-do = delayed
+def do(*args, **kwargs):
+    """deprecated, please use ``dask.delayed.delayed``"""
+    warnings.warn("`dask.delayed.do` is deprecated, please use "
+                  "`dask.delayed.delayed` instead")
+    return delayed(*args, **kwargs)
 
 
 def compute(*args, **kwargs):
-    """Evaluate more than one ``Delayed`` at once.
-
-    Note that the only difference between this function and
-    ``dask.base.compute`` is that this implicitly wraps python objects in
-    ``Delayed``, allowing for collections of dask objects to be computed.
-
-    Examples
-    --------
-    >>> a = delayed(1)
-    >>> b = a + 2
-    >>> c = a + 3
-    >>> compute(b, c)  # Compute both simultaneously
-    (3, 4)
-    >>> compute(a, [b, c])  # Works for lists of Delayed
-    (1, [3, 4])
-    """
-    args = [delayed(a) for a in args]
+    """deprecated, please use ``dask.compute``"""
+    warnings.warn("`dask.delayed.compute` is deprecated, please use "
+                  "`dask.compute` instead")
     return base.compute(*args, **kwargs)
 
 
@@ -303,14 +305,16 @@ class Delayed(base.Base):
 
     Equivalent to the output from a single key in a dask graph.
     """
-    __slots__ = ('_key', '_dasks', '_length')
+    __slots__ = ('_key', 'dask', '_length')
     _finalize = staticmethod(first)
     _default_get = staticmethod(threaded.get)
     _optimize = staticmethod(lambda d, k, **kwds: d)
 
-    def __init__(self, key, dasks, length=None):
+    def __init__(self, key, dsk, length=None):
         self._key = key
-        self._dasks = dasks
+        if type(dsk) is list:  # compatibility with older versions
+            dsk = sharedict.merge(*dsk)
+        self.dask = dsk
         self._length = length
 
     def __getstate__(self):
@@ -320,10 +324,6 @@ class Delayed(base.Base):
         for k, v in zip(self.__slots__, state):
             setattr(self, k, v)
 
-    @property
-    def dask(self):
-        return merge(*self._dasks)
-
     @property
     def key(self):
         return self._key
@@ -388,28 +388,28 @@ class Delayed(base.Base):
     _get_unary_operator = _get_binary_operator
 
 
-def call_function(func, args, kwargs, pure=False, nout=None):
+def call_function(func, func_token, args, kwargs, pure=False, nout=None):
     dask_key_name = kwargs.pop('dask_key_name', None)
     pure = kwargs.pop('pure', pure)
 
     if dask_key_name is None:
-        name = '%s-%s' % (funcname(func), tokenize(func, *args,
-                                                   pure=pure, **kwargs))
+        name = '%s-%s' % (funcname(func),
+                          tokenize(func_token, *args, pure=pure, **kwargs))
     else:
         name = dask_key_name
 
-    args, dasks = unzip(map(to_task_dasks, args), 2)
+    args, dasks = unzip(map(to_task_dask, args), 2)
+    dsk = sharedict.merge(*dasks)
     if kwargs:
-        dask_kwargs, dasks2 = to_task_dasks(kwargs)
-        dasks = dasks + (dasks2,)
+        dask_kwargs, dsk2 = to_task_dask(kwargs)
+        dsk.update(dsk2)
         task = (apply, func, list(args), dask_kwargs)
     else:
         task = (func,) + args
 
-    dasks = flat_unique(dasks)
-    dasks.append({name: task})
-    nout = nout if nout and nout > 1 else None
-    return Delayed(name, dasks, length=nout)
+    dsk.update_with_key({name: task}, key=name)
+    nout = nout if nout is not None else None
+    return Delayed(name, dsk, length=nout)
 
 
 class DelayedLeaf(Delayed):
@@ -425,12 +425,8 @@ class DelayedLeaf(Delayed):
     def dask(self):
         return {self._key: self._obj}
 
-    @property
-    def _dasks(self):
-        return [self.dask]
-
     def __call__(self, *args, **kwargs):
-        return call_function(self._obj, args, kwargs,
+        return call_function(self._obj, self._key, args, kwargs,
                              pure=self._pure, nout=self._nout)
 
 
@@ -443,11 +439,12 @@ class DelayedAttr(Delayed):
         self._key = key
 
     @property
-    def _dasks(self):
-        return [{self._key: (getattr, self._obj._key, self._attr)}] + self._obj._dasks
+    def dask(self):
+        dsk = {self._key: (getattr, self._obj._key, self._attr)}
+        return sharedict.merge(self._obj.dask, (self._key, dsk))
 
     def __call__(self, *args, **kwargs):
-        return call_function(methodcaller(self._attr), (self._obj,) + args, kwargs)
+        return call_function(methodcaller(self._attr), self._attr, (self._obj,) + args, kwargs)
 
 
 for op in [operator.abs, operator.neg, operator.pos, operator.invert,
