@@ -1,14 +1,13 @@
 from __future__ import absolute_import, division, print_function
 
-from itertools import count
+import re
 from operator import getitem
 
-from .compatibility import zip_longest
+from .compatibility import unicode
 
 from .core import add, inc  # noqa: F401
 from .core import (istask, get_dependencies, subs, toposort, flatten,
-                   reverse_dict, ishashable, preorder_traversal)
-from .rewrite import END
+                   reverse_dict, ishashable)
 
 
 def cull(dsk, keys):
@@ -57,7 +56,23 @@ def cull(dsk, keys):
     return out, dependencies
 
 
-def fuse(dsk, keys=None, dependencies=None):
+def default_fused_keys_renamer(keys):
+    """Create new keys for fused tasks"""
+    typ = type(keys[0])
+    if typ is str or typ is unicode:
+        names = [key_split(x) for x in keys[:0:-1]]
+        names.append(keys[0])
+        return '-'.join(names)
+    elif (typ is tuple and len(keys[0]) > 0 and
+          isinstance(keys[0][0], (str, unicode))):
+        names = [key_split(x) for x in keys[:0:-1]]
+        names.append(keys[0][0])
+        return ('-'.join(names),) + keys[0][1:]
+    else:
+        return None
+
+
+def fuse(dsk, keys=None, dependencies=None, rename_fused_keys=True):
     """ Return new dask graph with linear sequence of tasks fused together.
 
     If specified, the keys in ``keys`` keyword argument are *not* fused.
@@ -71,14 +86,23 @@ def fuse(dsk, keys=None, dependencies=None):
     dependencies: dict, optional
         {key: [list-of-keys]}.  Must be a list to provide count of each key
         This optional input often comes from ``cull``
+    rename_fused_keys: bool or func, optional
+        Whether to rename the fused keys with ``default_fused_keys_renamer``
+        or not.  Renaming fused keys can keep the graph more understandable
+        and comprehensive, but it comes at the cost of additional processing.
+        If False, then the top-most key will be used.  For advanced usage, a
+        func is also accepted, ``new_key = rename_fused_keys(fused_key_list)``.
 
     Examples
     --------
     >>> d = {'a': 1, 'b': (inc, 'a'), 'c': (inc, 'b')}
     >>> dsk, dependencies = fuse(d)
     >>> dsk # doctest: +SKIP
+    {'a-b-c': (inc, (inc, 1)), 'c': 'a-b-c'}
+    >>> dsk, dependencies = fuse(d, rename_fused_keys=False)
+    >>> dsk # doctest: +SKIP
     {'c': (inc, (inc, 1))}
-    >>> dsk, dependencies = fuse(d, keys=['b'])
+    >>> dsk, dependencies = fuse(d, keys=['b'], rename_fused_keys=False)
     >>> dsk  # doctest: +SKIP
     {'b': (inc, 1), 'c': (inc, 'b')}
 
@@ -131,12 +155,25 @@ def fuse(dsk, keys=None, dependencies=None):
             chain.append(child)
         chains.append(chain)
 
-    dependencies = dict((k, set(v)) for k, v in dependencies.items())
+    dependencies = {k: set(v) for k, v in dependencies.items()}
+
+    if rename_fused_keys is True:
+        key_renamer = default_fused_keys_renamer
+    elif rename_fused_keys is False:
+        key_renamer = None
+    else:
+        key_renamer = rename_fused_keys
 
     # create a new dask with fused chains
     rv = {}
     fused = set()
+    aliases = set()
+    is_renamed = False
     for chain in chains:
+        if key_renamer is not None:
+            new_key = key_renamer(chain)
+            is_renamed = (new_key is not None and new_key not in dsk and
+                          new_key not in rv)
         child = chain.pop()
         val = dsk[child]
         while chain:
@@ -147,11 +184,28 @@ def fuse(dsk, keys=None, dependencies=None):
             fused.add(child)
             child = parent
         fused.add(child)
-        rv[child] = val
-
+        if is_renamed:
+            rv[new_key] = val
+            rv[child] = new_key
+            dependencies[new_key] = dependencies[child]
+            dependencies[child] = {new_key}
+            aliases.add(child)
+        else:
+            rv[child] = val
     for key, val in dsk.items():
         if key not in fused:
             rv[key] = val
+    if aliases:
+        for key, deps in dependencies.items():
+            for old_key in deps & aliases:
+                new_key = rv[old_key]
+                deps.remove(old_key)
+                deps.add(new_key)
+                rv[key] = subs(rv[key], old_key, new_key)
+        if keys is not None:
+            for key in aliases - keys:
+                del rv[key]
+                del dependencies[key]
     return rv, dependencies
 
 
@@ -305,250 +359,6 @@ def functions_of(task):
     return funcs
 
 
-def dealias(dsk, keys=None, dependencies=None):
-    """ Remove aliases from dask
-
-    Removes and renames aliases using ``inline``. Optional ``keys`` keyword
-    argument protects keys from being deleted. This is useful to protect keys
-    that would be expected by a scheduler. If not provided, all inlined aliases
-    are removed.
-
-    Examples
-    --------
-    >>> dsk = {'a': (range, 5),
-    ...        'b': 'a',
-    ...        'c': 'b',
-    ...        'd': (sum, 'c'),
-    ...        'e': 'd',
-    ...        'f': (inc, 'd')}
-
-    >>> dealias(dsk)  # doctest: +SKIP
-    {'a': (range, 5),
-     'd': (sum, 'a'),
-     'f': (inc, 'd')}
-
-    >>> dsk = {'a': (range, 5),
-    ...        'b': 'a'}
-    >>> dealias(dsk)  # doctest: +SKIP
-    {'a': (range, 5)}
-
-    >>> dealias(dsk, keys=['a', 'b'])  # doctest: +SKIP
-    {'a': (range, 5),
-     'b': 'a'}
-    """
-    keys = keys or set()
-    if not isinstance(keys, set):
-        keys = set(keys)
-
-    if not dependencies:
-        dependencies = {k: get_dependencies(dsk, k)
-                        for k in dsk}
-
-    aliases = set(k for k, task in dsk.items() if
-                  ishashable(task) and task in dsk)
-
-    dsk2 = inline(dsk, aliases, inline_constants=False)
-    for k in aliases.difference(keys):
-        del dsk2[k]
-    return dsk2
-
-
-def equivalent(term1, term2, subs=None):
-    """Determine if two terms are equivalent, modulo variable substitution.
-
-    Equivalent to applying substitutions in `subs` to `term2`, then checking if
-    `term1 == term2`.
-
-    If a subterm doesn't support comparison (i.e. `term1 == term2` errors),
-    returns `False`.
-
-    Parameters
-    ----------
-    term1, term2 : terms
-    subs : dict, optional
-        Mapping of substitutions from `term2` to `term1`
-
-    Examples
-    --------
-    >>> from operator import add
-    >>> term1 = (add, 'a', 'b')
-    >>> term2 = (add, 'x', 'y')
-    >>> subs = {'x': 'a', 'y': 'b'}
-    >>> equivalent(term1, term2, subs)
-    True
-    >>> subs = {'x': 'a'}
-    >>> equivalent(term1, term2, subs)
-    False
-    """
-
-    # Quick escape for special cases
-    head_type = type(term1)
-    if type(term2) != head_type:
-        # If terms aren't same type, fail
-        return False
-    elif head_type not in (tuple, list):
-        # For literals, just compare
-        try:
-            # `is` is tried first, to allow objects that don't implement `==`
-            # to work for cases where term1 is term2. If `is` returns False,
-            # and `==` errors, then the only thing we can do is return False.
-            return term1 is term2 or term1 == term2
-        except:
-            return False
-
-    pot1 = preorder_traversal(term1)
-    pot2 = preorder_traversal(term2)
-    subs = {} if subs is None else subs
-
-    for t1, t2 in zip_longest(pot1, pot2, fillvalue=END):
-        if t1 is END or t2 is END:
-            # If terms aren't same length: fail
-            return False
-        elif ishashable(t2) and t2 in subs:
-            val = subs[t2]
-        else:
-            val = t2
-        try:
-            if t1 is not t2 and t1 != val:
-                return False
-        except:
-            return False
-    return True
-
-
-def dependency_dict(dsk):
-    """Create a dict matching ordered dependencies to keys.
-
-    Examples
-    --------
-    >>> from operator import add
-    >>> dsk = {'a': 1, 'b': 2, 'c': (add, 'a', 'a'), 'd': (add, 'b', 'a')}
-    >>> dependency_dict(dsk)    # doctest: +SKIP
-    {(): ['a', 'b'], ('a', 'a'): ['c'], ('b', 'a'): ['d']}
-    """
-
-    dep_dict = {}
-    for key in dsk:
-        deps = tuple(get_dependencies(dsk, key, True))
-        dep_dict.setdefault(deps, []).append(key)
-    return dep_dict
-
-
-def _possible_matches(dep_dict, deps, subs):
-    deps2 = []
-    for d in deps:
-        v = subs.get(d, None)
-        if v is not None:
-            deps2.append(v)
-        else:
-            return []
-    deps2 = tuple(deps2)
-    return dep_dict.get(deps2, [])
-
-
-def _sync_keys(dsk1, dsk2, dsk2_topo):
-    dep_dict1 = dependency_dict(dsk1)
-    subs = {}
-
-    for key2 in toposort(dsk2):
-        deps = tuple(get_dependencies(dsk2, key2, True))
-        # List of keys in dsk1 that have terms that *may* match key2
-        possible_matches = _possible_matches(dep_dict1, deps, subs)
-        if possible_matches:
-            val2 = dsk2[key2]
-            for key1 in possible_matches:
-                val1 = dsk1[key1]
-                if equivalent(val1, val2, subs):
-                    subs[key2] = key1
-                    break
-    return subs
-
-
-def sync_keys(dsk1, dsk2):
-    """Return a dict matching keys in `dsk2` to equivalent keys in `dsk1`.
-
-    Parameters
-    ----------
-    dsk1, dsk2 : dict
-
-    Examples
-    --------
-    >>> from operator import add, mul
-    >>> dsk1 = {'a': 1, 'b': (add, 'a', 10), 'c': (mul, 'b', 5)}
-    >>> dsk2 = {'x': 1, 'y': (add, 'x', 10), 'z': (mul, 'y', 2)}
-    >>> sync_keys(dsk1, dsk2)   # doctest: +SKIP
-    {'x': 'a', 'y': 'b'}
-    """
-
-    return _sync_keys(dsk1, dsk2, toposort(dsk2))
-
-
-def merge_sync(dsk1, dsk2):
-    """Merge two dasks together, combining equivalent tasks.
-
-    If a task in `dsk2` exists in `dsk1`, the task and key from `dsk1` is used.
-    If a task in `dsk2` has the same key as a task in `dsk1` (and they aren't
-    equivalent tasks), then a new key is created for the task in `dsk2`. This
-    prevents name conflicts.
-
-    Parameters
-    ----------
-    dsk1, dsk2 : dict
-        Variable names in `dsk2` are replaced with equivalent ones in `dsk1`
-        before merging.
-
-    Returns
-    -------
-    new_dsk : dict
-        The merged dask.
-    key_map : dict
-        A mapping between the keys from `dsk2` to their new names in `new_dsk`.
-
-    Examples
-    --------
-    >>> from operator import add, mul
-    >>> dsk1 = {'a': 1, 'b': (add, 'a', 10), 'c': (mul, 'b', 5)}
-    >>> dsk2 = {'x': 1, 'y': (add, 'x', 10), 'z': (mul, 'y', 2)}
-    >>> new_dsk, key_map = merge_sync(dsk1, dsk2)
-    >>> new_dsk     # doctest: +SKIP
-    {'a': 1, 'b': (add, 'a', 10), 'c': (mul, 'b', 5), 'z': (mul, 'b', 2)}
-    >>> key_map     # doctest: +SKIP
-    {'x': 'a', 'y': 'b', 'z': 'z'}
-
-    Conflicting names are replaced with auto-generated names upon merging.
-
-    >>> dsk1 = {'a': 1, 'res': (add, 'a', 1)}
-    >>> dsk2 = {'x': 1, 'res': (add, 'x', 2)}
-    >>> new_dsk, key_map = merge_sync(dsk1, dsk2)
-    >>> new_dsk     # doctest: +SKIP
-    {'a': 1, 'res': (add, 'a', 1), 'merge_1': (add, 'a', 2)}
-    >>> key_map     # doctest: +SKIP
-    {'x': 'a', 'res': 'merge_1'}
-    """
-
-    dsk2_topo = toposort(dsk2)
-    sd = _sync_keys(dsk1, dsk2, dsk2_topo)
-    new_dsk = dsk1.copy()
-    for key in dsk2_topo:
-        if key in sd:
-            new_key = sd[key]
-        else:
-            if key in dsk1:
-                new_key = next(merge_sync.names)
-            else:
-                new_key = key
-            sd[key] = new_key
-        task = dsk2[key]
-        for a, b in sd.items():
-            task = subs(task, a, b)
-        new_dsk[new_key] = task
-    return new_dsk, sd
-
-
-# store the name iterator in the function
-merge_sync.names = ('merge_%d' % i for i in count(1))
-
-
 def fuse_selections(dsk, head1, head2, merge):
     """Fuse selections with lower operation.
 
@@ -615,3 +425,59 @@ def fuse_getitem(dsk, func, place):
     """
     return fuse_selections(dsk, getitem, func,
                            lambda a, b: tuple(b[:place]) + (a[2], ) + tuple(b[place + 1:]))
+
+
+# Defining `key_split` (used by `default_fused_keys_renamer`) in utils.py
+# results in messy circular imports, so define it here instead.
+hex_pattern = re.compile('[a-f]+')
+
+
+def key_split(s):
+    """
+    >>> key_split('x')
+    u'x'
+    >>> key_split('x-1')
+    u'x'
+    >>> key_split('x-1-2-3')
+    u'x'
+    >>> key_split(('x-2', 1))
+    'x'
+    >>> key_split("('x-2', 1)")
+    u'x'
+    >>> key_split('hello-world-1')
+    u'hello-world'
+    >>> key_split(b'hello-world-1')
+    u'hello-world'
+    >>> key_split('ae05086432ca935f6eba409a8ecd4896')
+    'data'
+    >>> key_split('<module.submodule.myclass object at 0xdaf372')
+    u'myclass'
+    >>> key_split(None)
+    'Other'
+    >>> key_split('x-abcdefab')  # ignores hex
+    u'x'
+    """
+    if type(s) is bytes:
+        s = s.decode()
+    if type(s) is tuple:
+        s = s[0]
+    try:
+        words = s.split('-')
+        if not words[0][0].isalpha():
+            result = words[0].lstrip("'(\"")
+        else:
+            result = words[0]
+        for word in words[1:]:
+            if word.isalpha() and not (len(word) == 8 and
+                                       hex_pattern.match(word) is not None):
+                result += '-' + word
+            else:
+                break
+        if len(result) == 32 and re.match(r'[a-f0-9]{32}', result):
+            return 'data'
+        else:
+            if result[0] == '<':
+                result = result.strip('<>').split()[0].split('.')[-1]
+            return result
+    except Exception:
+        return 'Other'
